@@ -18,7 +18,8 @@
 from cubicweb.server import hook
 from cubicweb.selectors import is_instance
 
-IGNORE_FILES = set( ('README', 'series', '.hgignore') )
+IGNORE_FILES = set( ('README', 'series', 'status', '.hgignore', '.hgtags') )
+
 
 class LinkOrCreatePatchOp(hook.DataOperationMixIn, hook.Operation):
     containercls = list
@@ -27,7 +28,7 @@ class LinkOrCreatePatchOp(hook.DataOperationMixIn, hook.Operation):
         for entity in self.get_data():
             patch = None
             # if the revision is a renaming another one which is linked to the
-            # patch (take care DeletedVersionContent has not vc_rename relation)
+            # patch (take care DeletedVersionContent has no vc_rename relation)
             if getattr(entity, 'vc_rename', None) and entity.vc_rename[0].patch:
                 patch = entity.vc_rename[0].patch
             else:
@@ -35,9 +36,13 @@ class LinkOrCreatePatchOp(hook.DataOperationMixIn, hook.Operation):
                 branches = set(x.branch for x in entity.rev.parent_revision)
                 for branch in branches:
                     parent = entity.previous_version(branch)
-                    if parent is not None and parent.patch:
-                        patch = parent.patch
-                        break
+                    if parent is not None:
+                        if (entity.__regid__ == 'DeletedVersionContent' and
+                            getattr(parent, 'reverse_vc_rename', None)):
+                            return # skip file deleted because of renaming
+                        if parent.patch:
+                            patch = parent.patch
+                            break
             # create a new patch entity if no one found or if found a closed
             # patch and this is not a file deletion
             if patch is None or (not patch.is_active() and entity.__regid__ == 'VersionContent'):
@@ -47,12 +52,7 @@ class LinkOrCreatePatchOp(hook.DataOperationMixIn, hook.Operation):
             # patch file is being removed: mark it on the patch entity if still
             # active
             if entity.__regid__ == 'DeletedVersionContent' and patch.is_active():
-                if self.session.added_in_transaction(patch.eid):
-                    self.session.execute('SET X in_state S WHERE X eid %(x)s, S name %(state)s',
-                                         {'x': patch.eid, 'state': 'deleted'})
-                    # patch.cw_adapt_to('IWorkflowable').change_state('deleted')
-                else:
-                    patch.cw_adapt_to('IWorkflowable').fire_transition('file deleted')
+                MarkPatchAsDeletedOp.get_instance(self.session).add_data(patch)
 
 
 class LinkOrCreatePatchHook(hook.Hook):
@@ -77,6 +77,8 @@ class SearchPatchStateInstructionOp(hook.DataOperationMixIn,
     """search magic word in revision's commit message:
 
         <patch path> ready for review
+        reject <patch path>
+        fold <patch path>
 
     When found, the patch will be marked as pending review. You can put multiple
     instructions like this, one per line.
@@ -85,18 +87,39 @@ class SearchPatchStateInstructionOp(hook.DataOperationMixIn,
     def precommit_event(self):
         for rev in self.get_data():
             # search patches among files modified by the revision
-            candidates = {}
+            readycandidates = {}
+            rejectcandidates = {}
             for vc in rev.reverse_from_revision:
-                if vc.patch and vc.patch.cw_adapt_to('IWorkflowable').state == 'in-progress':
-                    candidates[vc.file.path] = vc.patch
+                if not vc.patch:
+                    continue
+                pstate = vc.patch.cw_adapt_to('IWorkflowable').state
+                if pstate == 'in-progress' and vc.__regid__ == 'VersionContent':
+                    readycandidates[vc.file.path] = vc.patch
+                elif (pstate in ('in-progress', 'pending-review', 'deleted') and
+                      vc.__regid__ == 'DeletedVersionContent'):
+                    rejectcandidates[vc.file.path] = vc.patch
             # search for instruction
             for line in rev.description.splitlines():
                 words = line.strip().split()
-                if words[0] in candidates and words[1:4] == ['ready', 'for', 'review']:
-                    patch = candidates.pop(words[0])
+                if not len(words) >= 2:
+                    continue
+                if words[0] in readycandidates and words[1:4] == ['ready', 'for', 'review']:
+                    patch = readycandidates.pop(words[0])
                     patch.cw_adapt_to('IWorkflowable').fire_transition(
                         'ask review', 'review asked by %s in %s' % (
                             rev.author, rev.changeset or rev.revision))
+                elif words[0] == 'reject' and words[1] in rejectcandidates:
+                    patch = rejectcandidates.pop(words[1])
+                    patch.cw_adapt_to('IWorkflowable').fire_transition(
+                        'reject', 'rejected by %s in %s' % (
+                            rev.author, rev.changeset or rev.revision))
+                    MarkPatchAsDeletedOp.get_instance(self.session)._container.remove(patch)
+                elif words[0] == 'fold' and words[1] in rejectcandidates:
+                    patch = rejectcandidates.pop(words[1])
+                    patch.cw_adapt_to('IWorkflowable').fire_transition(
+                        'fold', 'folded by %s in %s' % (
+                            rev.author, rev.changeset or rev.revision))
+                    MarkPatchAsDeletedOp.get_instance(self.session)._container.remove(patch)
 
 
 class SearchAppliedPatchOp(hook.DataOperationMixIn,
@@ -125,6 +148,8 @@ class SearchAppliedPatchOp(hook.DataOperationMixIn,
             # search for instruction
             for line in rev.description.splitlines():
                 words = line.strip().split()
+                if not words:
+                    continue
                 if words[-1] in candidates and words[-3:-1] == ['applied', 'patch']:
                     patch = candidates.pop(words[-1])
                     patch.cw_adapt_to('IWorkflowable').fire_transition('accept')
@@ -146,6 +171,18 @@ class RevisionAdded(hook.Hook):
             SearchAppliedPatchOp.get_instance(self._cw).add_data(self.entity)
 
 
+# use single last operation to be executed after all vcreview related operations
+class MarkPatchAsDeletedOp(hook.DataOperationMixIn, hook.SingleLastOperation):
+    def precommit_event(self):
+        for patch in self.get_data():
+            if self.session.added_in_transaction(patch.eid):
+                self.session.execute('SET X in_state S WHERE X eid %(x)s, S name %(state)s',
+                                     {'x': patch.eid, 'state': 'deleted'})
+                # patch.cw_adapt_to('IWorkflowable').change_state('deleted')
+            else:
+                patch.cw_adapt_to('IWorkflowable').fire_transition('file deleted')
+
+
 from cubes.nosylist import hooks as nosylist
 nosylist.S_RELS |= set(('has_activity',))
 nosylist.O_RELS |= set(('patch_repository',))
