@@ -26,17 +26,30 @@ class FakeRequest(object):
     def add_css(self, file): pass
 
 class DiffParserTC(TestCase):
-    def test_insert_point(self):
+    def transform(self, file):
+        data = open(self.datapath(file)).read()
+        trdata = TransformData(data, 'text/x-diff', 'utf-8')
+        trdata.appobject = mock_object(_cw=FakeRequest())
+        return ENGINE.convert(trdata, 'text/annotated-html').data
+
+    def setUp(self):
         def insert_point_cb(ipid, trdata, w):
             w('<hr class="%s"/>' % ipid)
             self.ipid = ipid
         ENGINE.add_transform(Diff2HTMLTransform(insert_point_cb))
-        data = open(self.datapath('pytestgc.diff')).read()
-        trdata = TransformData(data, 'text/x-diff', 'utf-8')
-        trdata.appobject = mock_object(_cw=FakeRequest())
-        htmldata = ENGINE.convert(trdata, 'text/annotated-html')
+
+    def tearDown(self):
+        ENGINE.remove_transform(Diff2HTMLTransform.name)
+
+    def test_insert_point_1(self):
+        htmldata = self.transform('pytestgc.diff')
         self.assertEqual(self.ipid, '4')
-        self.assertTrue(htmldata.data.startswith('<div class="text/x-diff">'))
+        self.assertTrue(htmldata.startswith('<div class="text/x-diff">'))
+
+    def test_insert_point_2(self):
+        htmldata = self.transform('nonregr1.diff')
+        self.assertEqual(self.ipid, '1')
+        self.assertTrue(htmldata.startswith('<div class="text/x-diff">'))
 
 if __name__ == '__main__':
     unittest_main()
