@@ -56,13 +56,23 @@ class PatchCreationHooksTC(VCSRepositoryTC):
         return vcsrepo
 
     def test_imported_patches(self):
-
         patches = self.execute('Patch X ORDERBY X')
+
         byfile = {}
+        bypatch = {}
         for patch in patches.entities():
             files = patch.patch_files()
-            self.assertEqual(len(files), 1)
-            byfile.setdefault(iter(files).next(), []).append(patch)
+            if len(files) == 1:
+                byfile.setdefault(iter(files).next(), []).append(patch)
+            else:
+                bypatch[patch] = files
+        self.assertEqual(len(byfile), 3, byfile)
+        # bypatch check we're correctly following renaming
+        self.assertEqual(len(bypatch), 1)
+        self.assertEqual(len(patches), 6)
+
+        folded = bypatch.keys()[0]
+        self.assertEqual(folded.cw_adapt_to('IWorkflowable').state, 'folded')
 
         self.assertEqual(len(byfile['patch1.diff']), 2)
         patch1_1 = byfile['patch1.diff'][0]
@@ -82,13 +92,13 @@ class PatchCreationHooksTC(VCSRepositoryTC):
         self.assertEqual(patch3_1.cw_adapt_to('IWorkflowable').state, 'deleted')
         self.assertEqual(revnums(patch3_1), [3, 4])
         patch3_2 = byfile['patch3.diff'][1]
-        self.assertEqual(patch3_2.cw_adapt_to('IWorkflowable').state, 'pending-review')
-        self.assertEqual(revnums(patch3_2), [7, 10])
-
-        self.assertEqual(len(patches), 5)
+        history = patch3_2.cw_adapt_to('IWorkflowable').workflow_history
+        self.assertEqual(history[-1].previous_state.name, 'pending-review')
+        self.assertEqual(history[-1].new_state.name, 'rejected')
+        self.assertEqual(revnums(patch3_2), [7, 10, 15])
 
         self.assertTrue(patch3_2.nosy_list)
-        self.assertEqual(len(MAILBOX), 1)
+        self.assertEqual(len(MAILBOX), 2)
         self.assertEqual(MAILBOX[0].subject, '[%s] patch %s is now in state "pending-review"'
                          % (patch3_2.repository.dc_title(), patch3_2.dc_title()))
         self.assertEqual(MAILBOX[0].recipients, ['admin@cwo'])
@@ -98,6 +108,17 @@ cubicweb changed status from <in-progress> to <pending-review> for entity
 
 review asked by Sylvain Th\xe9nault <sylvain.thenault@logilab.fr> in 9cf50f15d9a7
 
+url: http://testing.fr/cubicweb/patch/%s\n''' % patch3_2.eid)
+
+        self.assertEqual(MAILBOX[1].subject, '[%s] patch %s is now in state "rejected"'
+                         % (patch3_2.repository.dc_title(), patch3_2.dc_title()))
+        self.assertEqual(MAILBOX[1].recipients, ['admin@cwo'])
+        self.assertMultiLineEqual(MAILBOX[1].content, u'''
+cubicweb changed status from <pending-review> to <rejected> for entity
+'patch3.diff'
+
+rejected by Sylvain Th\xe9nault <sylvain.thenault@logilab.fr> in a5c0568f81db
+
 url: http://testing.fr/cubicweb/patch/%s\n''' % patch3_2.eid)
 
         MAILBOX[:] = ()
@@ -119,9 +140,13 @@ url: http://testing.fr/cubicweb/task/%s\n''' % done.eid)
         MAILBOX[:] = ()
         done.cw_adapt_to('IWorkflowable').fire_transition('done')
         self.commit()
-        patch3_2.cw_adapt_to('IWorkflowable')
         self.assertEqual(MAILBOX, [])
 
+        # force state
+        patch3_2.cw_adapt_to('IWorkflowable').change_state('pending-review')
+        self.commit()
+
+        MAILBOX[:] = ()
         # fake revision from the main repo of our patch repo
         revision = mock_object(repository=self.vcsrepo,
                                description='applied patch patch3.diff',
@@ -135,7 +160,7 @@ url: http://testing.fr/cubicweb/task/%s\n''' % done.eid)
         self.assertEqual(patch3_2.cw_adapt_to('IWorkflowable').state, 'applied')
         self.assertEqual(patch3_2.applied_at[0].eid, patch3_2.revisions[-1].rev.eid)
 
-        self.assertEqual(len(MAILBOX), 1)
+        self.assertEqual(len(MAILBOX), 1, MAILBOX)
         self.assertEqual(MAILBOX[0].subject, '[%s] patch %s is now in state "applied"'
                          % (patch3_2.repository.dc_title(), patch3_2.dc_title()))
         self.assertEqual(MAILBOX[0].recipients, ['admin@cwo'])
